@@ -55,7 +55,7 @@ short del;
 /* calling routine to free it.						*/
 /*----------------------------------------------------------------------*/
 
-char *evaluate_expr(oparamptr ops)
+char *evaluate_expr(oparamptr ops, objinstptr pinst)
 {
    if (ops->type != XC_EXPR) return NULL;
    return strdup(ops->parameter.expr);
@@ -207,7 +207,7 @@ stringpart *nextstringpart(stringpart *strptr, objinstptr thisinst)
    stringpart *nextptr = strptr->nextpart;
 
    if (strptr->type == PARAM_START)
-      nextptr = linkstring(thisinst, strptr);
+      nextptr = linkstring(thisinst, strptr, TRUE);
    else if (strptr->type == PARAM_END) {
       strptr->nextpart = NULL;
 
@@ -222,6 +222,24 @@ stringpart *nextstringpart(stringpart *strptr, objinstptr thisinst)
    return nextptr;
 }
 
+/*----------------------------------------------------------------------*/
+/* Same as the above routine, but don't recompute expression parameters	*/
+/* when encountered.  Use the previously generated result.		*/
+/*----------------------------------------------------------------------*/
+
+stringpart *nextstringpartnorecompute(stringpart *strptr, objinstptr thisinst)
+{
+   stringpart *nextptr = strptr->nextpart;
+
+   if (strptr->type == PARAM_START)
+      nextptr = linkstring(thisinst, strptr, FALSE);
+   else if (strptr->type == PARAM_END) {
+      strptr->nextpart = NULL;
+      if (strptr->data.string != (u_char *)NULL)
+	 freelabel((stringpart *)(strptr->data.string));
+   }
+   return nextptr;
+}
 
 /*----------------------------------------------------------------------*/
 /* Remove a string part from the string					*/
@@ -318,9 +336,14 @@ stringpart *mergestring(stringpart *firststr)
 
 /*----------------------------------------------------------------------*/
 /* Link a parameter to a string						*/
+/* If compute_exprs is TRUE, then we should recompute any expression	*/
+/* parameters encountered.  If FALSE, then we assume that all		*/
+/* expressions have been computed previously, and may use the recorded	*/
+/* instance value.							*/
 /*----------------------------------------------------------------------*/
 
-stringpart *linkstring(objinstptr localinst, stringpart *strstart)
+stringpart *linkstring(objinstptr localinst, stringpart *strstart,
+	Boolean compute_exprs)
 {
    char *key;
    stringpart *tmpptr, *nextptr = NULL;
@@ -364,15 +387,27 @@ stringpart *linkstring(objinstptr localinst, stringpart *strstart)
       if (ops->type == XC_INT) {
 	 promote->data.string = (char *)malloc(13);
 	 sprintf(promote->data.string, "%12d", ops->parameter.ivalue);
+         nextptr = promote;
       }
       else if (ops->type == XC_FLOAT) {
 	 promote->data.string = (char *)malloc(13);
 	 sprintf(promote->data.string, "%g", (double)(ops->parameter.fvalue));
+         nextptr = promote;
+      }
+      else {	/* ops->type == XC_EXPR */
+	 oparamptr ips;
+	 if (!compute_exprs && (ips = match_instance_param(localinst, key))
+		!= NULL && (ips->type == XC_STRING)) {
+	    nextptr = ips->parameter.string;
+	    /* This could probably be done better. . . hack to keep  */
+	    /* free() from operating twice on the same memory block. */
+	    promote->data.string = (char *)malloc(1);
+	 }
+	 else {
+	    promote->data.string = evaluate_expr(ops, localinst);
+            nextptr = promote;
+	 }
       }
-      else
-	 promote->data.string = evaluate_expr(ops);
-
-      nextptr = promote;
    }
    else
       nextptr = ops->parameter.string;
@@ -1350,7 +1385,7 @@ void UDrawString(labelptr drawlabel, int passcolor, objinstptr localinst)
 
        pos = 0;
        for (strptr = drawlabel->string; strptr != NULL;
-		strptr = nextstringpart(strptr, localinst)) {
+		strptr = nextstringpartnorecompute(strptr, localinst)) {
 
 	  /* All segments other than text cancel any	*/
 	  /* existing overline/underline in effect.	*/
